
var JRTCSDK = require("../../JRTCSDK-Wechat.js");

const {
  JRTCClient,
  JRTCClientInitParam,
  JRTCClientLoginParam,
  ClientState,
  JRTCMediaDevice,
  RenderType,
  JRTCGuest,
  JRTCCallCenterInitParam,
  JRTCCallCenterCallParam,
  GuestCallStateChangeType,
  JRTCTracking,
  VideoDefinition
} = JRTCSDK;

Page({
  context: {
    client: null,
    clientCallback: null,
    clientState: ClientState.NOT_INIT,

    mediaDevice: null,
    mediaDeviceCallback: null,

    guest: null,
    guestCallback: null,

    localCanvas: null,
    remoteCanvas: null,
  },

  data: {
    isInitialized: false,
    //client的环境参数
    clientEnv: {
      server: 'http://192.168.4.171:19993',
      appKey: '05c97ce0d0a84d2592334093',
      token: '',
      tokenType: 'jrtc_access',
      zantongClient: false
    },
    //房间的环境参数
    roomEnv: {
      server: 'http://192.168.4.171:9000'
    },
    accountSettings: {
      userId:'',
      displayName:''
    },

    clientStateText: '未登录',
    isLogin: false,

    maxResolutionMap: ['360p', '720p', '1080p'],
    securityTypeMap: ['不加密', 'SRTP', 'SM4'],
    videoEncodeTypeMap: ['H264', 'AV1'],
    audioEncodeTypeMap: ['OPUS', 'PCMA', 'PCMU'],
    renderTypeMap: ['FullContent', 'FullScreen'],

    // 渲染模式
    renderType: RenderType.RENDER_FULL_CONTENT,
    mediaSettings: {
      isSelfDefined: false,
      maxResolution: 0,
      svcResolution: "1 90 70 180 180 360 480 720 1350",
      captureResolution: 1,
      frameRate: 24
    },
    roomSettings: {
      sendMessage: {
        type: '',
        content: '',
        userId: '',
      },
      participants: [],
    },

    callAgentSettings: {
      telNumber: '',
      agentUserId: '',
      displayName: '',
      autoRecord: true,
      maxResolution: 0,
      svcResolution: "1 90 70 180 180 360 480 720 1350",
      heartbeatTime: 20,
      heartbeatTimeout: 60,
      videoEncodeType: 0,
      audioEncodeType: 0,
      extraInfo: '',
      securityType: 0,
      captureWidth: 360,
      captureHeight: 640,
      frameRate: 24,
      videoDefinition: 0,
      vipPower:0 //呼叫权重
    },

    queueInfo: '',

    publishing: false, // 是否正在推流

    callState: 'idle',

    callStateText: '空闲',

    isTalking: false,

    isUrgent: false,

    localVolume: 0,
    remoteVolume: 0,

    errorCodeDetail: [],
    errorCodeNodes: [{
      name: 'div',
      attrs: {
        class: 'nodesInfo',
      },
      children: []
    }],

    sendMessageInfo: {
      sendUserId: '',
      sendContent: '',
      sendoperatorId: -1
    },
    dialogInfo: {
      dialogShow: false,
      buttons: [{ text: '挂断' }, { text: '接听' }],
      message: '',
      title: '',
    },
    toastInfo: {
      textToast: false,
      hideTextToast: false,
      contentToast: '',
    },

    // dialogShow: false,
    // buttons: [{text: '取消'}, {text: '确定'}],

    //权限
    permissions: ['scope.camera', 'scope.record'],
    /**
    * dialog
    */
    isCancel: false,
    content: undefined,
  },

  onReady: function () {
    //初始化缓存数据
    var signalServer = wx.getStorageSync('signalServer')
    var roomServer = wx.getStorageSync('roomServer')
    var appKey = wx.getStorageSync('appKey')
    var token = wx.getStorageSync('token')
    var tokenType = wx.getStorageSync('tokenType')
    if (!isEmptyStr(signalServer)) {
      this.setData({
        "clientEnv.server": signalServer
      })
    }
    if (!isEmptyStr(roomServer)) {
      this.setData({
        "roomEnv.server": roomServer
      })
    }
    if (!isEmptyStr(appKey)) {
      this.setData({
        "clientEnv.appKey": appKey
      })
    }
    if (!isEmptyStr(token)) {
      this.setData({
        "clientEnv.token": token
      })
    }
    if (!isEmptyStr(tokenType)) {
      this.setData({
        "clientEnv.tokenType": tokenType
      })
    }
  },

  onLoad: function () {
  },

  onUnload: function () {
    this.sdkUnInit();
  },

  onShow: function () {
  },

  onHide: function () {
  },

  sdkInit() {
    console.debug("sdkInit");
    this.context.clientState = ClientState.NOT_INIT;
    const clientInitParam = new JRTCClientInitParam();
    clientInitParam.appName = "JCCSample-Wechat";
    clientInitParam.server = this.data.clientEnv.server;
    clientInitParam.appKey = this.data.clientEnv.appKey;
    this.context.clientState = ClientState.IDLE;
    this.initClientCallback();
    this.context.client = JRTCClient.create(this.context.clientCallback, clientInitParam);
    JRTCTracking.setChannel("Wechat channel");

    this.initMediaDeviceCallback();
    this.context.mediaDevice = JRTCMediaDevice.create(this.context.mediaDeviceCallback, {});
    // 设置摄像头采集
    this.context.mediaDevice.setCameraProperty(360, 640, 24);

    this.initGuestCallback();
    let initParam = new JRTCCallCenterInitParam();
    initParam.roomServer = this.data.roomEnv.server;
    initParam.protocol = 'RTMP';
    this.context.guest = JRTCGuest.create(this.context.client, this.context.mediaDevice, initParam, this.context.guestCallback);
    if (this.context.client != null) {
      this.setData({
        isInitialized: true,
      })
    }
    //申请权限，加载到设置中,可查看到申请的权限
    this.requestPermission();
    //设置缓存数据
    wx.setStorageSync('signalServer', this.data.clientEnv.server);
    wx.setStorageSync('roomServer', this.data.roomEnv.server);
    wx.setStorageSync('appKey', this.data.clientEnv.appKey);
  },


  sdkUnInit() {
    this.setData({
      isInitialized: false,
    })
    if (this.data.isLogin) {
      this.context.client.logout();
    }
    if (this.context.client != null) {
      JRTCClient.destroy();
      this.context.client = null;
    }
    if (this.context.mediaDevice != null) {
      JRTCMediaDevice.destroy();
      this.context.mediaDevice = null;
    }
    if (this.context.guest != null) {
      JRTCGuest.destroy();
      this.context.guest = null;
    }
  },

  sdkInitHandle() {
    if (this.data.isInitialized) {
      this.sdkUnInit();
    } else {
      this.sdkInit();
    }
  },

  handleLogin() {
    if (this.data.isLogin) {
      this.context.client.logout();
      if (this.data.callState != 'idle' && this.data.callState != 'termed') {
        this.context.guest.term();
      }
    } else {
      if (isEmptyStr(this.data.accountSettings.userId)) {
        this.openTextToast('帐号不能为空');
        return;
      }
      let loginParam = new JRTCClientLoginParam();
      loginParam.token = this.data.clientEnv.token;
      loginParam.tokenType = this.data.clientEnv.tokenType;
      this.context.client.setServer(this.data.clientEnv.server);
      this.context.client.setAppKey(this.data.clientEnv.appKey);
      this.context.client.setDisplayName(this.data.accountSettings.displayName);
      this.context.client.login(this.data.accountSettings.userId, '123456', loginParam);
      this.showLoading('登录中...');
      wx.setStorageSync('token', this.data.clientEnv.token);
      wx.setStorageSync('tokenType', this.data.clientEnv.tokenType);
    }
  },

  sendOnLineMsg() {
    let operatorId = this.context.client.sendOnlineMessage(this.data.sendMessageInfo.sendContent, this.data.sendMessageInfo.sendUserId);
    // this.setData({
    //   sendMessageInfo:{
    //     sendoperatorId: operatorId
    //   }
    // })
  },

  queryGroup() {
    this.context.guest.queryAllGroups();
  },

  bindMaxResolutionChange: function (e) {
    this.setData({
      ['mediaSettings.maxResolution']: parseInt(e.detail.value)
    })
  },

  bindCaptureResolutionChange: function (e) {
    this.setData({
      ['mediaSettings.captureResolution']: parseInt(e.detail.value)
    })
  },

  bindSecurityTypeChange: function (e) {
    this.setData({
      ['callAgentSettings.securityType']: parseInt(e.detail.value)
    })
  },
  bindVideoEncodeTypeChange: function (e) {
    this.setData({
      ['callAgentSettings.videoEncodeType']: parseInt(e.detail.value)
    })
  },
  bindAudioEncodeTypeChange: function (e) {
    this.setData({
      ['callAgentSettings.audioEncodeType']: parseInt(e.detail.value)
    })
  },
  bindRenderChange: function (e) {
    this.setData({
      renderType: parseInt(e.detail.value)
    })
  },
  checkSelfDefinedChange() {
    this.setData({
      ['mediaSettings.isSelfDefined']: !this.data.mediaSettings.isSelfDefined
    })
  },
  checkEnableRecordChange (){
    this.setData({
      ['callAgentSettings.autoRecord']: !this.data.callAgentSettings.autoRecord
    })
  },

  handleCall(callType) {
    let that = this;
    let isOneToOneCall = false;
    let telNumberOrAgentUserId;
    const operation = callType.currentTarget.dataset.op;
    if (operation == 'call') {
      if (isEmptyStr(that.data.callAgentSettings.telNumber)) {
        that.openTextToast('请输入业务号');
        return;
      }
      isOneToOneCall = false;
      telNumberOrAgentUserId = that.data.callAgentSettings.telNumber;
    } else if (operation == 'oneToOneCall') {
      if (isEmptyStr(that.data.callAgentSettings.agentUserId)) {
        that.openTextToast('请输入座席ID');
        return;
      }
      isOneToOneCall = true;
      telNumberOrAgentUserId = that.data.callAgentSettings.agentUserId;
    }
    //设置媒体参数
    if (that.data.mediaSettings.isSelfDefined) {
      that.data.callAgentSettings.videoDefinition = 0;
      if (that.data.mediaSettings.captureResolution == 0) {
        that.data.callAgentSettings.captureWidth = 360;
        that.data.callAgentSettings.captureHeight = 640;
      } else if (that.data.mediaSettings.captureResolution == 1) {
        that.data.callAgentSettings.captureWidth = 480;
        that.data.callAgentSettings.captureHeight = 720;
      } else if (that.data.mediaSettings.captureResolution == 2) {
        that.data.callAgentSettings.captureWidth = 720;
        that.data.callAgentSettings.captureHeight = 1080;
      }
      if (!that.data.callAgentSettings.frameRate | that.data.callAgentSettings.frameRate <=0){
        this.data.callAgentSettings.frameRate = 24;
      }
      that.data.callAgentSettings.svcResolution = this.data.mediaSettings.svcResolution;
    } else {
      if (that.data.mediaSettings.maxResolution == 0) {
        that.data.callAgentSettings.captureWidth = 360;
        that.data.callAgentSettings.captureHeight = 640;
        that.data.callAgentSettings.videoDefinition = VideoDefinition.FLUENCY_FRAME_HIGH;
      } else if (that.data.mediaSettings.maxResolution == 1) {
        that.data.callAgentSettings.captureWidth = 480;
        that.data.callAgentSettings.captureHeight = 720;
        that.data.callAgentSettings.videoDefinition = VideoDefinition.STANDARD_FRAME_HIGH;
      }else if (that.data.mediaSettings.maxResolution == 2) {
        that.data.callAgentSettings.captureWidth = 720;
        that.data.callAgentSettings.captureHeight = 1080;
        that.data.callAgentSettings.videoDefinition = VideoDefinition.HIGH_FRAME_HIGH;
      }
      that.data.callAgentSettings.frameRate = 24;
      that.data.callAgentSettings.maxResolution = that.data.mediaSettings.maxResolution;
    }
    that.context.mediaDevice.setCameraProperty(that.data.callAgentSettings.captureWidth,that.data.callAgentSettings.captureHeight,that.data.callAgentSettings.frameRate);

    that.checkPermission(that.data.permissions).then((res) => {
      if (!res.result) {
        that.openPermissionSetting(res.scope);
      } else {
        JRTCTracking.setBusinessId("请设置businessId");
        let callParam = new JRTCCallCenterCallParam();
        callParam.autoRecord = that.data.callAgentSettings.autoRecord;
        callParam.svcResolution = that.data.callAgentSettings.svcResolution;
        callParam.maxResolution = that.data.callAgentSettings.maxResolution;
        callParam.maxFrameRate = that.data.callAgentSettings.frameRate;
        callParam.securityType = that.data.callAgentSettings.securityType;
        callParam.extraInfo = that.data.callAgentSettings.extraInfo;
        callParam.videoEncodeType = that.data.callAgentSettings.videoEncodeType;
        callParam.audioEncodeType = that.data.callAgentSettings.audioEncodeType;
        callParam.heartbeatTime = that.data.callAgentSettings.heartbeatTime;
        callParam.heartbeatTimeout = that.data.callAgentSettings.heartbeatTimeout;
        callParam.videoDefinition = that.data.callAgentSettings.videoDefinition;
        callParam.vipPower = parseInt(that.data.callAgentSettings.vipPower);
        if (isOneToOneCall) {
          if (that.context.guest.oneToOneCall(telNumberOrAgentUserId, callParam)) {
            //this.showLoading('呼叫等待中...');
          } else {
            that.openTextToast('呼叫失败');
          }
        } else {
          if (that.context.guest.call(telNumberOrAgentUserId, callParam)) {
            //this.showLoading('呼叫等待中...');
          } else {
            that.openTextToast('呼叫失败');
          }
        }
      }
    });
  },

  requestUrgent() {
    this.context.guest.requestUrgent();
  },

  term() {
    this.context.guest.term();
  },

  startCameraVideo(that) {
    console.debug("startCameraVideo");
    let viewId = 'local-stream';
    // 或者传入viewId和document
    that.context.mediaDevice.startCameraVideo(that.data.renderType, viewId, that, true).then((canvas) => {
      that.context.localCanvas = canvas;
    }).catch((e) => {
      console.log('摄像头未打开', e);
      this.openTextToast('本地视频渲染失败');
    });
  },

  stopCameraVideo() {
    if (this.context.localCanvas != null) {
      this.context.mediaDevice.stopVideo(this.context.localCanvas);
    }
  },

  switchCamera() {
    this.context.mediaDevice.switchCamera();
  },

  snapshot() {
    let that = this;
    that.context.localCanvas.snapshot('raw').then(res => {
      const fileManager = wx.getFileSystemManager();
      let imgBase64 = res;
      //将base64数据写入到相册
      var filePath = wx.env.USER_DATA_PATH + '/test.png';
      fileManager.writeFile({
        filePath: filePath,
        data: imgBase64,
        encoding: 'base64',
        success: res => {
          //保存图片到相册
          wx.saveImageToPhotosAlbum({
            filePath: filePath,
            success: function (data) {
              console.log(data);
              that.openTextToast('图片已保存到相册');
            },
            fail: function (err) {
              console.log(err);
              if (err.errMsg === "saveImageToPhotosAlbum:fail auth deny") {
                console.log("用户一开始拒绝了，我们想再次发起授权")
                that.openPermissionSetting('scope.writePhotosAlbum');
              }
            }
          })
        }, fail: err => {
          console.log(err)
        }
      })
    }).catch(e => {
      console.error(e);
      that.openTextToast('截图失败');
    });
  },


  getParticipants() {
    const parts = this.context.guest.getParticipants();
    this.data.roomSettings.participants = parts;
    console.log('parts: ', parts);
  },

  subscribe(participant, that) {

    that.context.guest.requestVideo(participant, { width: 100, height: 100 }).then((result) => {
      let viewId = 'remote-stream';
      let selector = '.live-player';
      console.log('requestVideo result', result);
      that.context.remoteCanvas = that.context.mediaDevice.startVideo(result, that.data.renderType, viewId, that, selector);
      if (!that.context.remoteCanvas) {
        this.openTextToast('视频流请求失败');
      }
    });
  },

  publish() {
    if (this.data.publishing) {
      return;
    }
    this.context.guest.enableUploadVideoStream(true);
    this.data.publishing = true;
    this.setData({
      publishing: true,
    });
  },

  unPublish() {
    if (!this.data.publishing) {
      return;
    }
    this.context.guest.enableUploadVideoStream(false);
    this.data.publishing = false;
    this.setData({
      publishing: false,
    });
  },

  sendMessage() {
    if (this.data.roomSettings.sendMessage.userId == '' || !this.data.roomSettings.sendMessage.userId) {
      this.context.guest.sendMessage(this.data.roomSettings.sendMessage.type, this.data.roomSettings.sendMessage.content, null);
    } else {
      this.context.guest.sendMessage(this.data.roomSettings.sendMessage.type, this.data.roomSettings.sendMessage.content, this.data.roomSettings.sendMessage.userId);
    }
  },

  onClientStatusChanged(state) {
    if (state == ClientState.LOGINING) {
      this.data.clientStateText = '登录中';
    } else if (state == ClientState.LOGINED) {
      this.data.clientStateText = '已登录';
    } else if (state == ClientState.LOGOUTING) {
      this.data.clientStateText = '登出中';
    } else {
      this.data.clientStateText = '未登录';
    }
  },

  onCallStatusChanged(type) {
    if (type == GuestCallStateChangeType.CALLING) {
      this.data.callState = 'calling';
      this.data.callStateText = '正在呼叫...';
    } else if (type == GuestCallStateChangeType.WAITING) {
      this.data.callState = 'waiting';
      this.data.callStateText = '呼叫等待中...';
    } else if (type == GuestCallStateChangeType.INCOMING) {
      this.data.callState = 'incoming';
      this.data.callStateText = '收到来电';
    } else if (type == GuestCallStateChangeType.TALKING) {
      this.data.callState = 'talking';
      this.data.callStateText = '通话中';
    } else if (type == GuestCallStateChangeType.TERMED) {
      this.data.callState = 'termed';
      this.data.callStateText = '通话结束';
    }
  },


  hasPermission(scope) {
    return new Promise((resolve, reject) => {
      wx.getSetting({
        success: res => {
          if (res.authSetting[scope]) {
            console.log(scope + '权限检测成功');
            resolve(true);
          } else {
            console.log(scope + '权限检测失败');
            resolve(false);
          }
        },
        fail: res => {
          reject(false);
        }
      })
    });
  },

  /**
   * 检测权限,返回检测结果
   */
  async checkPermission(scope) {
    let that = this;
    let isCheckSuccess = true;
    let result = {
      result: false,
      scope: ''
    };
    if (Array.isArray(scope)) {
      for (let i = 0; i < scope.length; i++) {
        if (!isCheckSuccess) {
          break;
        }
        result.scope = scope[i];
        await that.hasPermission(scope[i]).then((res) => {
          if (res) {
            isCheckSuccess = true;
          } else {
            isCheckSuccess = false;
          }
        });
      }
    }
    result.result = isCheckSuccess;
    return Promise.resolve(result);
  },

  /**
   * 弹框打开设置，申请权限
   */
  openPermissionSetting(scope) {
    let that = this;
    let content = '';
    switch (scope) {
      case 'scope.camera':
        content = '摄像头';
        break;
      case 'scope.record':
        content = '麦克风';
        break;
      case 'scope.writePhotosAlbum':
        content = '添加到相册';
        break;
    }
    wx.showModal({
      title: '申请权限',
      content: "需要使用" + content + "该功能，请前往设置打开权限",
      success(res) {
        if (res.confirm) {
          console.log('用户点击确定')
          wx.openSetting({
            success(res) {
              if (scope === 'scope.writePhotosAlbum') {
                if (res.authSetting[scope]) {
                  that.showDialog(false, '权限申请成功,请重新截图');
                }
              }
            },
            fail(err) {
              console.log('失败', err)
            }
          })
        } else if (res.cancel) {
          console.log('用户点击取消')
        }
      }
    })
  },

  //申请权限，加载到设置中,可查看到申请的权限
  requestPermission() {
    this.data.permissions.forEach(function (value) {
      wx.authorize({
        scope: value,
        success: res => {
          console.log('授权成功');
        },
        fail: res => {
          console.log('授权失败');
        }
      });
    })
  },

  isScreenShareParticipant(participant) {
    if (participant.userId === 'SCREEN__') {
      return true;
    }
    return false;
  },

  initClientCallback() {
    let that = this;
    this.context.clientCallback = {
      onLogin: (result, reason) => {
        console.log('onLogin result: ', result, ' reason: ', reason);
        that.hideLoading();
        if (!result) {
          this.openTextToast('登录失败');
        }
      },
      onLogout: (reason) => {
        console.log('onLogout reason: ', reason);
        that.hideLoading();
        that.term();
      },
      onClientStateChanged: (state, oldState) => {
        console.debug('onClientStateChanged: %d => %d', oldState, state);
        that.context.clientState = state;
        var isLogin = (state == ClientState.LOGINED || state == ClientState.LOGINING);
        that.onClientStatusChanged(state);
        that.setData({
          isLogin: isLogin,
          clientStateText: this.data.clientStateText
        });
      },
      onSDKEvent: (event) => {
        if (event.errorCode && event.errorCode.length > 0 && event.errorCodeDetail && event.errorCodeDetail.length > 0) {
          let content = "错误码:" + event.errorCode + "错误描述:" + event.errorCodeDetail;
          that.data.errorCodeDetail.push({ name: 'p', attrs: {}, children: [{ type: 'text', text: content }] });
          that.setData({
            errorCodeNodes: [{
              name: 'div',
              attrs: {
                class: 'nodesInfo',
              },
              children: that.data.errorCodeDetail
            }]
          });
        }
      },
      onOnlineMessageReceived: (message, userId) => {
        this.openTextToast('收到来自 ' + userId + ' 发送的在线消息: ' + message);
      },
      onOnlineMessageSendResult: (result, operatorId) => {
        this.openTextToast('发送' + result ? '成功' : '失败' + ' operatorId: ' + operatorId);
      }
    }
  },

  initMediaDeviceCallback() {
    this.context.mediaDeviceCallback = {
      onVolumeChanged: (volume, userId) => {
        if (userId == null || userId == this.data.accountSettings.userId) {
          this.data.localVolume = volume;
        } else {
          this.data.remoteVolume = volume;
        }
        console.log('sample onVolumeChanged: %s, volume: %s', userId, volume);
      },
      onCameraUpdate: () => {

      }
    }
  },

  initGuestCallback() {
    let that = this;
    that.context.guestCallback = {
      onGetAllGroups: (result, groups) => {
        if (result && groups && groups.length > 0) {
          that.showDialog(false, JSON.stringify(groups));
        }
      },
      onCallStateChanged: (type, incomingType, inviter, reason) => {
        that.onCallStatusChanged(type);
        var isTalking = type == GuestCallStateChangeType.TALKING;
        that.setData({
          callState: that.data.callState,
          callStateText: that.data.callStateText,
          isTalking: isTalking
        });

        if (type == GuestCallStateChangeType.INCOMING) {
          if (inviter) {
            let displayName = inviter.userId;
            if (inviter.displayName) {
              displayName = inviter.displayName;
            }
            let title = '收到' + displayName + '来电';
            let message = '';
            if (inviter.extraInfo) {
              message = + 'extraInfo: ' + inviter.extraInfo;
            }
            this.setData({
              dialogInfo: {
                dialogShow: true,
                message: message,
                title: title,
                buttons: [{ text: '挂断' }, { text: '接听' }],
              }
            })
          }
        } else if (type == GuestCallStateChangeType.TALKING) {
          Promise.all(
            [that.startCameraVideo(that)]
          ).then(() => {
            that.setData({
              publishing: true,
            });
          });
          that.data.queueInfo = '';
          that.setData({
            queueInfo: ''
          });
        } else if (type == GuestCallStateChangeType.TERMED) {
          that.stopCameraVideo();
          that.data.publishing = false;
          that.data.queueInfo = '';
          that.setData({
            publishing: false,
            queueInfo: '',
            isUrgent: false
          });
          that.showDialog(false, '通话结束,原因: ' + reason);
        }
      },

      onCallQueueCount: (count, time) => {
        console.log('onCallQueueCount count: ', count, time);
        that.setData({
          queueInfo: ' 排队人数: ' + count + ' 等待时长: ' + time
        });
        if (this.data.callState == 'calling') {
          that.setData({
            callState: 'waiting'
          });
        }
      },
      onCallPropertyChanged: (propChangeParam) => {
        console.log('onCallPropertyChanged propChangeParam: ', propChangeParam);
      },
      onMemberJoin: (participant) => {
        console.log('onMemberJoin participant: ', participant);
        this.openTextToast('成员：' + participant.userId + '加入');
        that.getParticipants();

        if (participant.userId == 'MERGED__') {
          that.subscribe(participant, that);
        }
      },
      onMemberLeave: (participant) => {
        this.openTextToast('成员：' + participant.userId + '离开');
      },
      onMemberUpdate: (participant, changeParam) => {
        console.log('onMemberUpdate participant: ', participant);
      },

      onUrgentResultResponse: (agree) => {
        console.log('onUrgentResultResponse agree : ', agree);
        that.setData({
          isUrgent: agree
        })
        if (agree) {
          this.openTextToast("加急成功");
        } else {
          this.openTextToast("加急失败");
        }
      },
      onHoldStateChanged: (hold) => {

      },
      onCallTypeChanged: (callType) => {

      },
      onMessageReceived: (content, contentType, messageType, fromUserId) => {

        if (contentType == 'InviteScreenShare') {
          let desc = 'WeChat无法进行协作';
          let descMap = { reason_code: 0, reason_desc: desc };
          this.context.guest.sendMessage('Decline', JSON.stringify(descMap), fromUserId);
          return;
        }
        let message = "收到消息:" + content + " contentType:" + contentType + " fromUserId:" + fromUserId;
        this.openTextToast(message);
      },

      onOnewayVideoChanged: (turnOn) => {

      },
      onDeliveryAbort: (isShutDown, deliveryUri, reason) => {

      },
      onCallForwarding: () => {

      },
      onNotifyMessageReceived: (notifyMessage, fromUserId) => {
        console.log('onNotifyMessageReceived notifyMessage: ', notifyMessage, fromUserId);
      },
      onSignRequest: (userId, extraInfo) => {
        let message = "收到 " + userId + "签名请求 " + extraInfo;
        this.openTextToast(message);
        setTimeout(() => {
          this.context.guest.responseSignResult(false, userId, 'Wechat不支持签名');
        }, 2000);
      },
    }
  },

  tapDialogButton(e) {
    if (e.detail.index == 0) {
      this.context.guest.term();
    } else if (e.detail.index == 1) {
      this.context.guest.answer();
    }
    this.setData({
      dialogInfo: {
        dialogShow: false
      }
    })
  },

  openTextToast(content) {
    this.setData({
      toastInfo: {
        textToast: true,
        contentToast: content,
      }
    });
    setTimeout(() => {
      this.setData({
        toastInfo: {
          hideTextToast: true,
        }
      });
      setTimeout(() => {
        this.setData({
          toastInfo: {
            textToast: false,
            hideTextToast: false,
            contentToast: '',
          }
        });
      }, 300);
    }, 3000);
  },

  handleSimpleInput: function (ev) {
    var _a;
    var path = ev.currentTarget.dataset['path'];
    this.setData((_a = {},
      _a[path] = ev.detail.value,
      _a));
  },

  showDialog(isCancel, content) {
    wx.showModal({
      showCancel: isCancel,
      content: content
    })
  },

  showLoading(content) {
    wx.showLoading({
      title: content
    })
  },
  hideLoading() {
    wx.hideLoading();
  }
});

function isEmptyStr(str) {
  if (str == '') {
    return true
  }
  return false
}