
var JRTCSDK = require("../../JRTCSDK-Wechat.js");

const {
  JRTCClient,
  JRTCClientInitParam,
  JRTCClientLoginParam,
  ClientState,
  JRTCMediaDevice,
  RenderType,
  JRTCRoom,
  JRTCRoomInitParam,
  JRTCRoomJoinParam,
  StompSocketAuthType,
  RoomState,
  RemoteRecordState,
  JRTCTracking,
  JRTCRecordRemoteParam,
  VideoDefinition
} = JRTCSDK;

Page({

  context: {
    client: null,
    clientCallback: null,
    clientState: ClientState.NOT_INIT,

    mediaDevice: null,
    mediaDeviceCallback: null,

    room: null,
    roomCallback: null,
    roomState: RoomState.IDLE,

    localCanvas: null,
    remoteCanvas: null,
  },

  data: {
    isInitialized: false,
    //client的环境参数
    clientEnv: {
      server: 'http://192.168.4.171:19993',
      appKey: '05c97ce0d0a84d2592334093',
      token: '',
      tokenType: 'jrtc_access',
      zanTongClient: false
    },

    //房间的环境参数
    roomEnv: {
      server: 'http://192.168.4.171:9000'
    },

    clientStateText: '未登录',
    isLogin: false,

    cameraList: [],

    microphoneList: [],

    localAuido: null,

    remoteAudio: null,

    roomStatus: 'idle',

    roomStateText: '',

    isJoined: false,

    maxResolutionMap: ['360p', '720p', '1080p'],
    securityTypeMap: ['不加密', 'SRTP', 'SM4',],
    videoEncodeTypeMap: ['H264', 'AV1'],
    audioEncodeTypeMap: ['OPUS', 'PCMA', 'PCMU'],
    renderTypeMap: ['FullContent', 'FullScreen'],

    // 渲染模式
    renderType: RenderType.RENDER_FULL_CONTENT,
    mediaSettings: {
      isSelfDefined: false,
      maxResolution: 0,
      svcResolution: "1 90 70 180 180 360 480 720 1350",
      captureResolution: 1,
      frameRate: 24
    },

    joinSettings: {
      userId: '',
      displayName: '',
      roomId: '',
      password: '',
      capacity: 6,
      svcResolution: '1 90 70 180 180 360 480 720 1350',
      enableRecord: true,
      maxResolution: 0,
      uploadLocalVideo: true,
      uploadLocalAudio: true,
      wholeRatio: 1.7777,
      securityType: 0,
      smoothMode: true,
      heartbeatTime: 20,
      heartbeatTimeout: 60,
      videoEncodeType: 0,
      audioEncodeType: 0,
      extraInfo: '',
      captureWidth: 360,
      captureHeight: 640,
      frameRate: 24,
      videoDefinition: 0,
    },

    roomSettings: {
      currentCamera: null,
      currentMic: null,
      remoteRecordSettings: {
        bitrate: 15,
        frameRate: 24,
        recordVideo: true,
        videoWidth: 640,
        videoHeight: 360,
        fileName: 'Wechat_record'
      },
      queryRoom: {
        roomId: undefined,
      },
      sendMessage: {
        type: '',
        content: '',
        userId: '',
      },
      participants: [],
      remoteParticipantList: [],

      screenShareUserId: '',
      recordState: RemoteRecordState.NONE,

      enableRecord: true,//默认可以开启录制
      enableStopRecord: false,
    },

    publishing: false, // 是否正在推流


    errorCodeDetail: [],
    errorCodeNodes: [{
      name: 'div',
      attrs: {
        class: 'nodesInfo',
      },
      children: []
    }],

    //权限
    permissions: ['scope.camera', 'scope.record'],
    /**
     * dialog
     */
    isCancel: false,
    content: undefined,

    sendMessageInfo: {
      sendUserId: '',
      sendContent: '',
      sendoperatorId: -1
    },
    toastInfo: {
      textToast: false,
      hideTextToast: false,
      contentToast: '',
    },
  },

  onReady() {
    //初始化缓存数据
    var signalServer = wx.getStorageSync('signalServer')
    var roomServer = wx.getStorageSync('roomServer')
    var appKey = wx.getStorageSync('appKey')
    var token = wx.getStorageSync('token')
    var tokenType = wx.getStorageSync('tokenType')
    if (!isEmptyStr(signalServer)) {
      this.setData({
        "clientEnv.server": signalServer
      })
    }
    if (!isEmptyStr(roomServer)) {
      this.setData({
        "roomEnv.server": roomServer
      })
    }
    if (!isEmptyStr(appKey)) {
      this.setData({
        "clientEnv.appKey": appKey
      })
    }
    if (!isEmptyStr(token)) {
      this.setData({
        "clientEnv.token": token
      })
    }
    if (!isEmptyStr(tokenType)) {
      this.setData({
        "clientEnv.tokenType": tokenType
      })
    }
  },
  onUnload: function () {
    this.sdkUnInit();
  },
  onShow: function () {
  },
  onHide: function () {
  },

  sdkInit() {
    console.debug("sdkInit");
    this.context.clientState = ClientState.NOT_INIT;
    const clientInitParam = new JRTCClientInitParam();
    clientInitParam.appName = "JCCSample-Wechat";
    clientInitParam.server = this.data.clientEnv.server;
    clientInitParam.appKey = this.data.clientEnv.appKey;
    this.context.clientState = ClientState.IDLE;
    this.initClientCallback();
    this.context.client = JRTCClient.create(this.context.clientCallback, clientInitParam);
    JRTCTracking.setChannel("请设置channel");

    this.initMediaDeviceCallback();
    this.context.mediaDevice = JRTCMediaDevice.create(this.context.mediaDeviceCallback, {});

    this.initRoomCallback();
    const param = new JRTCRoomInitParam();
    param.roomServer = this.data.roomEnv.server;
    param.protocol = 'RTMP';

    const paramStr = param.toString();
    console.log('param: ', paramStr);
    this.context.room = JRTCRoom.create(this.context.client, this.context.mediaDevice, param, this.context.roomCallback);
    if (this.context.client != null) {
      this.setData({
        isInitialized: true,
      })
    }
    //申请权限，加载到设置中,可查看到申请的权限
    this.requestPermission();
    //设置缓存数据
    wx.setStorageSync('signalServer', this.data.clientEnv.server);
    wx.setStorageSync('roomServer', this.data.roomEnv.server);
    wx.setStorageSync('appKey', this.data.clientEnv.appKey);
  },

  sdkUnInit() {
    this.setData({
      isInitialized: false,
    })
    if (this.data.isLogin) {
      this.context.client.logout();
    }
    if (this.context.client != null) {
      JRTCClient.destroy();
    }
    if (this.context.mediaDevice != null) {
      JRTCMediaDevice.destroy();
    }
    if (this.context.room != null) {
      JRTCRoom.destroy();
    }
  },

  sdkInitHandle() {
    if (this.data.isInitialized) {
      this.sdkUnInit();
    } else {
      this.sdkInit();
    }
  },

  loginHandle() {
    if (this.data.isLogin) {
      this.context.client.logout();
    } else {
      if (isEmptyStr(this.data.joinSettings.userId)) {
        this.openTextToast('帐号不能为空');
        return;
      }
      this.showLoading('登录中...');
      let loginParam = new JRTCClientLoginParam();
      loginParam.token = this.data.clientEnv.token;
      loginParam.tokenType = this.data.clientEnv.tokenType;
      loginParam.zanTongClient = this.data.clientEnv.zanTongClient;
      this.context.client.setDisplayName = this.data.joinSettings.displayName;
      this.context.client.login(this.data.joinSettings.userId, '123456', loginParam);
      wx.setStorageSync('token', this.data.clientEnv.token);
      wx.setStorageSync('tokenType', this.data.clientEnv.tokenType);
    }
  },

  sendOnLineMsg() {
    let operatorId = this.context.client.sendOnlineMessage(this.data.sendMessageInfo.sendContent, this.data.sendMessageInfo.sendUserId);
    // this.setData({
    //   sendMessageInfo:{
    //     sendoperatorId: operatorId
    //   }
    // })
  },

  /* page actions  */
  bindMaxResolutionChange: function (e) {
    this.setData({
      ['mediaSettings.maxResolution']: parseInt(e.detail.value)
    })
  },
  bindRenderChange: function (e) {
    this.setData({
      ['mediaSettings.renderType']: parseInt(e.detail.value)
    })
  },

  bindCaptureResolutionChange: function (e) {
    this.setData({
      ['mediaSettings.captureResolution']: parseInt(e.detail.value)
    })
  },
  bindSecurityTypeChange: function (e) {
    this.setData({
      ['joinSettings.securityType']: parseInt(e.detail.value)
    })
  },
  checkEnableRecordChange() {
    this.setData({
      ['joinSettings.enableRecord']: !this.data.joinSettings.enableRecord
    })
  },
  checkSelfDefinedChange() {
    this.setData({
      ['mediaSettings.isSelfDefined']: !this.data.mediaSettings.isSelfDefined
    })
  },
  checkSmoothModeChange() {
    this.setData({
      ['joinSettings.smoothMode']: !this.data.joinSettings.smoothMode
    })
  },
  checkLocalVideoChange() {
    this.setData({
      ['joinSettings.uploadLocalVideo']: !this.data.joinSettings.uploadLocalVideo
    })
  },

  checkLocalAudioChange() {
    this.setData({
      ['joinSettings.uploadLocalAudio']: !this.data.joinSettings.uploadLocalAudio
    })
  },
  bindVideoEncodeTypeChange: function (e) {
    this.setData({
      ['joinSettings.videoEncodeType']: parseInt(e.detail.value)
    })
  },
  bindAudioEncodeTypeChange: function (e) {
    this.setData({
      ['joinSettings.audioEncodeType']: parseInt(e.detail.value)
    })
  },

  startCamera() {
    console.log("startCamera");
    //this.context.mediaDevice.startCamera();
  },
  stopCamera() {
    console.log("stopCamera");
    //this.context.mediaDevice.stopCamera();
  },
  getCameras() {
    // this.cameras = this.mediaDevice.getCameras();
    // this.data.roomSettings.currentCamera = this.cameras[0];
  },
  startCameraVideo(that) {
    console.debug("startCameraVideo");
    let viewId = 'local-stream';
    // 或者传入viewId和document
    that.context.mediaDevice.startCameraVideo(that.renderType, viewId, that, true).then((canvas) => {
      that.context.localCanvas = canvas;
    }).catch((e) => {
      console.log('摄像头未打开', e);
      this.openTextToast('本地视频渲染失败');
    });

  },

  stopCameraVideo() {
    if (this.context.localCanvas != null) {
      this.context.mediaDevice.stopVideo(this.context.localCanvas);
    }
  },

  joinRoom() {
    if (!this.data.isLogin) {
      this.showDialog(false, '请先登录');
      return;
    }

    if (this.data.isJoined) {
      this.openTextToast('已入会');
      return;
    }

    if (isEmptyStr(this.data.joinSettings.roomId)) {
      this.openTextToast('房间号不能为空');
      return;
    }
    // 设置媒体参数
    if (this.data.mediaSettings.isSelfDefined) {
      this.data.joinSettings.videoDefinition = 0;
      if (this.data.mediaSettings.captureResolution == 0) {
        this.data.joinSettings.captureWidth = 360;
        this.data.joinSettings.captureHeight = 640;
      } else if (this.data.mediaSettings.captureResolution == 1) {
        this.data.joinSettings.captureWidth = 480;
        this.data.joinSettings.captureHeight = 720;
      } else if (this.data.mediaSettings.captureResolution == 2) {
        this.data.joinSettings.captureWidth = 720;
        this.data.joinSettings.captureHeight = 1080;
      }
      if (!this.data.joinSettings.frameRate | this.data.joinSettings.frameRate <= 0) {
        this.data.joinSettings.frameRate = 24;
      }
      this.data.joinSettings.svcResolution = this.data.mediaSettings.svcResolution;
    } else {
      if (this.data.mediaSettings.maxResolution == 0) {
        this.data.joinSettings.captureWidth = 360;
        this.data.joinSettings.captureHeight = 640;
        this.data.joinSettings.videoDefinition = VideoDefinition.FLUENCY_FRAME_HIGH;
      } else if (this.data.mediaSettings.maxResolution == 1) {
        this.data.joinSettings.captureWidth = 480;
        this.data.joinSettings.captureHeight = 720;
        this.data.joinSettings.videoDefinition = VideoDefinition.STANDARD_FRAME_HIGH;
      } else if (this.data.mediaSettings.maxResolution == 2) {
        this.data.joinSettings.captureWidth = 720;
        this.data.joinSettings.captureHeight = 1080;
        this.data.joinSettings.videoDefinition = VideoDefinition.HIGH_FRAME_HIGH;
      }
      this.data.joinSettings.frameRate = 24;
      this.data.joinSettings.maxResolution = this.data.mediaSettings.maxResolution;
    }
    // 设置摄像头采集
    this.context.mediaDevice.setCameraProperty(this.data.joinSettings.captureWidth, this.data.joinSettings.captureHeight, this.data.joinSettings.frameRate);
    this.checkPermission(this.data.permissions).then((res) => {
      if (!res.result) {
        this.openPermissionSetting(res.scope);
      } else {
        this.showLoading('加入房间中...');
        const param = new JRTCRoomJoinParam();
        param.enableRemoteRecord = true;
        param.uploadLocalAudio = true;
        param.uploadLocalVideo = true;

        param.capacity = this.data.joinSettings.capacity;
        param.password = this.data.joinSettings.password;
        param.svcResolution = this.data.joinSettings.svcResolution;
        param.maxResolution = this.data.joinSettings.maxResolution;
        param.maxFrameRate = this.data.joinSettings.frameRate;
        param.wholeRatio = this.data.joinSettings.wholeRatio;
        param.securityType = this.data.joinSettings.securityType;
        param.smoothMode = this.data.joinSettings.smoothMode;
        param.heartbeatTime = this.data.joinSettings.heartbeatTime;
        param.heartbeatTimeout = this.data.joinSettings.heartbeatTimeout;
        param.extraInfo = this.data.joinSettings.extraInfo;
        param.videoEncodeType = this.data.joinSettings.videoEncodeType;
        param.audioEncodeType = this.data.joinSettings.audioEncodeType;
        param.videoDefinition = this.data.joinSettings.videoDefinition;
        const paramStr = param.toString();
        this.context.room.join(this.data.joinSettings.roomId, param);
      }
    });
  },

  discoveryRoom() {
    let that = this;
    const roomId = that.data.roomSettings.queryRoom.roomId;
    console.log('discoveryRoom roomId: ', roomId);
    let appKey = that.data.clientEnv.appKey;
    wx.request({
      url: `${that.data.roomEnv.server}/room/discovery?roomId=${roomId}&appKey=${appKey}`,
      header: {
        'content-type': 'application/json' // 默认值
      }, success: function (res) {
        let data = res.data.data;
        if (data != null) {
          if (data.exist != null && data.exist) {
            that.openTextToast(`房间号：${roomId}已经存在`);
          } else {
            that.openTextToast(`房间号：${roomId}不存在`);
          }
        } else {
          that.openTextToast(`查询房间失败`);
        }
      }, fail(res) {
        that.openTextToast(`查询房间失败`);
      }
    })
  },

  leave() {
    if (this.data.isJoined) {
      this.context.room.leave();
    }
  },

  getParticipants() {
    const parts = this.context.room.getParticipants();
    this.data.roomSettings.participants = parts;
  },

  subscribe(participant, that) {
    this.context.room.requestVideo(participant, { width: 100, height: 100 }).then((result) => {
      let viewId = 'remote-stream';
      this.context.remoteCanvas = this.context.mediaDevice.startVideo(result, that.renderType, viewId, that);
      if (!this.context.remoteCanvas) {
        this.openTextToast('视频流请求失败');
      }
    });
  },

  publish() {
    if (this.data.publishing) {
      return;
    }
    this.context.room.enableUploadVideoStream(true);
    this.data.publishing = true;
    this.setData({
      publishing: true,
    });

  },

  unPublish() {
    if (!this.data.publishing) {
      return;
    }
    this.context.room.enableUploadVideoStream(false);
    this.data.publishing = false;
    this.setData({
      publishing: false,
    });
  },


  switchCamera() {
    this.context.mediaDevice.switchCamera();
  },

  snapshot() {
    let that = this;
    that.context.localCanvas.snapshot('raw').then(res => {
      const fileManager = wx.getFileSystemManager();
      let imgBase64 = res;
      //将base64数据写入到相册
      var filePath = wx.env.USER_DATA_PATH + '/test.png';
      fileManager.writeFile({
        filePath: filePath,
        data: imgBase64,
        encoding: 'base64',
        success: res => {
          //保存图片到相册
          wx.saveImageToPhotosAlbum({
            filePath: filePath,
            success: function (data) {
              console.log(data);
              that.openTextToast(`图片已保存到相册`);
            },
            fail: function (err) {
              console.log(err);
              if (err.errMsg === "saveImageToPhotosAlbum:fail auth deny") {
                console.log("用户一开始拒绝了，我们想再次发起授权")
                that.openPermissionSetting('scope.writePhotosAlbum');
              }
            }
          })
        }, fail: err => {
          console.log(err)
        }
      })
    }).catch(e => {
      console.error(e);
      this.openTextToast(`截图失败`);
    });
  },

  startRemoteRecord() {
    if (this.context.room.getRemoteRecordState() != RemoteRecordState.READY) {
      this.showDialog(false, '开启录制失败');
      return;
    }
    let param = new JRTCRecordRemoteParam();
    param.iBitrate = this.data.roomSettings.remoteRecordSettings.bitrate;
    param.frameRate = this.data.roomSettings.remoteRecordSettings.frameRate;
    param.recordVideo = this.data.roomSettings.remoteRecordSettings.recordVideo;
    param.videoWidth = this.data.roomSettings.remoteRecordSettings.videoWidth;
    param.videoHeight = this.data.roomSettings.remoteRecordSettings.videoHeight;
    param.fileName = this.data.roomSettings.remoteRecordSettings.fileName;
    this.context.room.enableRemoteRecord(true, param);
  },

  stopRemoteRecord() {
    this.context.room.enableRemoteRecord(false, {});
  },

  sendMessage() {
    if (this.data.roomSettings.sendMessage.userId == '' || !this.data.roomSettings.sendMessage.userId) {
      for (const part of this.data.roomSettings.participants) {
        if (part) {
          this.context.room.sendMessage(this.data.roomSettings.sendMessage.type, this.data.roomSettings.sendMessage.content, part.userId);
        }
      }
    } else {
      this.context.room.sendMessage(this.data.roomSettings.sendMessage.type, this.data.roomSettings.sendMessage.content, this.data.roomSettings.sendMessage.userId);
    }
  },

  hasPermission(scope) {
    return new Promise((resolve, reject) => {
      wx.getSetting({
        success: res => {
          if (res.authSetting[scope]) {
            console.log(scope + '权限检测成功');
            resolve(true);
          } else {
            console.log(scope + '权限检测失败');
            resolve(false);
          }
        },
        fail: res => {
          reject(false);
        }
      })
    });
  },

  /**
   * 检测权限,返回检测结果
   */
  async checkPermission(scope) {
    let that = this;
    let isCheckSuccess = true;
    let result = {
      result: false,
      scope: ''
    };
    if (Array.isArray(scope)) {
      for (let i = 0; i < scope.length; i++) {
        if (!isCheckSuccess) {
          break;
        }
        result.scope = scope[i];
        await that.hasPermission(scope[i]).then((res) => {
          if (res) {
            isCheckSuccess = true;
          } else {
            isCheckSuccess = false;
          }
        });
      }
    }
    result.result = isCheckSuccess;
    return Promise.resolve(result);
  },

  /**
   * 弹框打开设置，申请权限
   */
  openPermissionSetting(scope) {
    let that = this;
    let content = '';
    switch (scope) {
      case 'scope.camera':
        content = '摄像头';
        break;
      case 'scope.record':
        content = '麦克风';
        break;
      case 'scope.writePhotosAlbum':
        content = '添加到相册';
        break;
    }
    wx.showModal({
      title: '申请权限',
      content: "需要使用" + content + "该功能，请前往设置打开权限",
      success(res) {
        if (res.confirm) {
          console.log('用户点击确定')
          wx.openSetting({
            success(res) {
              if (scope === 'scope.writePhotosAlbum') {
                if (res.authSetting[scope]) {
                  that.showDialog(false, '权限申请成功,请重新截图');
                }
              }
            },
            fail(err) {
              console.log('失败', err)
            }
          })
        } else if (res.cancel) {
          console.log('用户点击取消')
        }
      }
    })
  },

  //申请权限，加载到设置中,可查看到申请的权限
  requestPermission() {
    this.data.permissions.forEach(function (value) {
      wx.authorize({
        scope: value,
        success: res => {
          console.log('授权成功');
        },
        fail: res => {
          console.log('授权失败');
        }
      });
    })
  },

  initClientCallback() {
    let that = this;
    this.context.clientCallback = {
      onLogin: (result, reason) => {
        console.log('onLogin result: ', result, ' reason: ', reason);
        if (!result) {
          that.openTextToast(`登录失败`);
        }
        that.hideLoading();
      },
      onLogout: (reason) => {
        console.log('onLogout reason: ', reason);
        that.leave();
      },
      onClientStateChanged: (state, oldState) => {
        console.log('onClientStateChanged state: ', state);
        that.context.clienstState = state;
        var isLogin = (state == ClientState.LOGINED || state == ClientState.LOGINING);
        that.onClientStatusChanged(state);
        that.setData({
          isLogin: isLogin,
          clientStateText: this.data.clientStateText
        });
      },
      onSDKEvent: (event) => {
        console.log('onSDKEvent event: ', event);
        if (event.errorCode && event.errorCode.length > 0 && event.errorCodeDetail && event.errorCodeDetail.length > 0) {
          let content = "错误码:" + event.errorCode + "错误描述:" + event.errorCodeDetail;
          that.data.errorCodeDetail.push({ name: 'p', attrs: {}, children: [{ type: 'text', text: content }] });
          that.setData({
            errorCodeNodes: [{
              name: 'div',
              attrs: {
                class: 'nodesInfo',
              },
              children: that.data.errorCodeDetail
            }]
          });
        }
      },
      onOnlineMessageReceived: (message, userId) => {
        this.openTextToast('收到来自 ' + userId + ' 发送的在线消息: ' + message);
      },
      onOnlineMessageSendResult: (result, operatorId) => {
        this.openTextToast('发送' + result ? '成功' : '失败' + ' operatorId: ' + operatorId);
      }
    }
  },

  initMediaDeviceCallback() {
    this.context.mediaDeviceCallback = {
      onVolumeChanged: (volume, userId) => {
        if (userId == null || userId == this.joinSettings.userId) {
          this.localVolume = volume / 100;
        } else {
          this.remoteVolume = volume / 100;
        }
        console.log('sample onVolumeChanged: %s, volume: %s', userId, volume);
      },
      onCameraUpdate: () => {

      },
      onAudioInputUpdate: () => {

      }
    }
  },


  initRoomCallback() {
    let that = this;
    this.context.roomCallback = {
      onJoin: (result, reason, roomId, room) => {
        console.log('onJoin result: ', result, reason, roomId, room);
        that.hideLoading();
        if (result) {
          Promise.all(
            [that.startCameraVideo(that)]
          ).then(() => {
            console.log('startCameraVideo then');
            that.data.publishing = true;
            that.data.isJoined = true;
            that.setData({
              publishing: true,
              isJoined: true,
            });
          });
        } else {
          that.showModal(false, '加入房间失败');
        }
      },
      onQuery: (operationId, result, reason, queryInfo) => {
        console.log('onQuery operationId : result: reason: queryInfo: ', operationId, result, reason, queryInfo);
        if (result) {
          that.showDialog(false, queryInfo.toString());
        } else {
          this.openTextToast(`房间信息查询失败`);
        }
      },
      onRoomStateChanged: (state, oldState) => {
        console.log('onRoomStateChange state:%s oldState:%s', state, oldState);
        that.data.roomState = state;
        that.data.isJoined = state == RoomState.JOINED;
        that.onRoomStatusChanged(state);
        that.setData({
          isJoined: this.data.isJoined,
          roomStateText: this.data.roomStateText
        });
      },

      onLeave: (reason, roomId) => {
        console.log('onLeave');
        that.data.publishing = false;
        that.data.isJoined = false;
        that.setData({
          publishing: false,
          isJoined: false,
          "roomSettings.enableRecord": false,
          "roomSettings.enableStopRecord": false
        });
      },
      onParticipantJoin: (participant) => {
        console.log('onParticipantJoin participant: ', participant);
        this.openTextToast('成员：' + participant.userId + '加入');
        that.getParticipants();
        if (participant.userId == 'MERGED__') {
          that.subscribe(participant, that);
        }
        // //更新录制状态
        // if (that.context.room.getRemoteRecordState() == RemoteRecordState.READY){
        //   this.setData({
        //     "roomSettings.enableRecord": true,
        //     "roomSettings.enableStopRecord": false
        //   })
        // }
      },
      onParticipantLeft: (participant) => {
        this.openTextToast('成员：' + participant.userId + '离开');
      },
      onParticipantUpdate: (participant, changeParam, room) => {
        console.log('onParticipantUpdate participant: ', participant, changeParam);
      },
      onRoomPropertyChanged: (param) => {
        console.log('onRoomPropertyChanged param: ', param);
        if (param.remoteRecordState) {
          that.data.roomSettings.recordState = that.context.room.getRemoteRecordState();
          that.data.roomSettings.enableRecord = that.data.roomSettings.recordState == RemoteRecordState.READY;
          that.data.roomSettings.enableStopRecord = that.data.roomSettings.recordState == RemoteRecordState.RUNNING;
          that.setData({
            "roomSettings.enableRecord": that.data.roomSettings.enableRecord,
            "roomSettings.enableStopRecord": that.data.roomSettings.enableStopRecord
          })
        }
      },

      onMessageReceived: (type, content, fromUserId) => {
        if (type == 'JCRemoteDesktop') {
          let desc = 'WeChat无法进行协作';
          let descMap = { reason_code: 0, reason_desc: desc };
          this.context.room.sendMessage('Decline', JSON.stringify(descMap), fromUserId);
          return;
        }

        console.log('onMessageReceived type: ', type, content, fromUserId);
        let message = "收到消息：" + content + " type：" + type + " fromUserId：" + fromUserId;
        this.openTextToast(message);
      }
    }
  },

  onClientStatusChanged(state) {
    if (state == ClientState.LOGINING) {
      this.data.clientStateText = '登录中';
    } else if (state == ClientState.LOGINED) {
      this.data.clientStateText = '已登录';
    } else if (state == ClientState.LOGOUTING) {
      this.data.clientStateText = '登出中';
    } else {
      this.data.clientStateText = '未登录';
    }
  },

  onRoomStatusChanged(roomState) {
    if (roomState == RoomState.IDLE) {
        this.data.roomStatus = 'idle';
        this.data.roomStateText = '空闲';
    } else if (roomState == RoomState.JOINING) {
        this.data.roomStatus = "joining";
        this.data.roomStateText = '正在加入房间...';
    } else if (roomState == RoomState.JOINED) {
        this.data.roomStatus = "joined";
        this.data.roomStateText = '已经加入房间';
    } else if (roomState == RoomState.LEAVING) {
        this.data.roomStatus = "leaving";
        this.data.roomStateText = '正在离开房间...';
    }
},

  getRoomStatus(roomState) {
    if (roomState == RoomState.IDLE) {
      return 'idle';
    } else if (roomState == RoomState.JOINING) {
      return 'joining'
    } else if (roomState == RoomState.JOINED) {
      return 'joined';
    } else if (roomState == RoomState.LEAVING) {
      return 'leaving'
    }
  },

  isScreenShareParticipant(participant) {
    if (participant.userId === 'SCREEN__') {
      return true;
    }
    return false;
  },

  clearStreamNode() {

  },

  openTextToast(content) {
    this.setData({
      toastInfo: {
        textToast: true,
        contentToast: content,
      }
    });
    setTimeout(() => {
      this.setData({
        toastInfo: {
          hideTextToast: true,
        }
      });
      setTimeout(() => {
        this.setData({
          toastInfo: {
            textToast: false,
            hideTextToast: false,
            contentToast: '',
          }
        });
      }, 300);
    }, 3000);
  },

  handleSimpleInput: function (ev) {
    var _a;
    var path = ev.currentTarget.dataset['path'];
    this.setData((_a = {},
      _a[path] = ev.detail.value,
      _a));
  },

  showDialog(isCancel, content) {
    wx.showModal({
      showCancel: isCancel,
      content: content
    })
  },

  showLoading(content) {
    wx.showLoading({
      title: content
    })
  },
  hideLoading() {
    wx.hideLoading();
  }

});

function isEmptyStr(str) {
  if (str == '') {
    return true
  }
  return false
}